"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProxy = exports.waitForServerReady = void 0;
var express_1 = __importDefault(require("express"));
var http_proxy_1 = __importDefault(require("http-proxy"));
var body_parser_1 = __importDefault(require("body-parser"));
var http = __importStar(require("http"));
var logger_1 = __importDefault(require("../../../common/logger"));
var stateHandler_1 = require("./stateHandler/stateHandler");
var hooks_1 = require("./hooks");
var tracer_1 = require("./tracer");
// Listens for the server start event
exports.waitForServerReady = function (server) {
    return new Promise(function (resolve, reject) {
        server.on('listening', function () { return resolve(server); });
        server.on('error', function () {
            return reject(new Error('Unable to start verification proxy server'));
        });
    });
};
// Get the Proxy we'll pass to the CLI for verification
exports.createProxy = function (config, stateSetupPath) {
    var app = express_1.default();
    var proxy = new http_proxy_1.default();
    logger_1.default.trace("Setting up state proxy with path: " + stateSetupPath);
    app.use(stateSetupPath, body_parser_1.default.json());
    app.use(stateSetupPath, body_parser_1.default.urlencoded({ extended: true }));
    hooks_1.registerBeforeHook(app, config, stateSetupPath);
    hooks_1.registerAfterHook(app, config, stateSetupPath);
    // Trace req/res logging
    if (config.logLevel === 'debug' || config.logLevel === 'trace') {
        logger_1.default.info('debug request/response logging enabled');
        app.use(tracer_1.createRequestTracer());
        app.use(tracer_1.createResponseTracer());
    }
    // Allow for request filtering
    if (config.requestFilter !== undefined) {
        app.use(config.requestFilter);
    }
    // Setup provider state handler
    app.post(stateSetupPath, stateHandler_1.createProxyStateHandler(config));
    // Proxy server will respond to Verifier process
    app.all('/*', function (req, res) {
        logger_1.default.debug("Proxying " + req.path);
        proxy.web(req, res, {
            changeOrigin: config.changeOrigin === true,
            secure: config.validateSSL === true,
            target: config.providerBaseUrl,
        });
    });
    return http.createServer(app).listen();
};
//# sourceMappingURL=proxy.js.map