"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.uuid = exports.fromProviderState = exports.arrayContaining = exports.url = exports.url2 = exports.nullValue = exports.includes = exports.date = exports.time = exports.timestamp = exports.datetime = exports.equal = exports.regex = exports.string = exports.number = exports.decimal = exports.integer = exports.boolean = exports.constrainedArrayLike = exports.atMostLike = exports.atLeastLike = exports.atLeastOneLike = exports.eachLike = exports.eachKeyLike = exports.like = void 0;
var ramda_1 = require("ramda");
var index_node_1 = __importDefault(require("../../native/index.node"));
/**
 * Value must match the given template
 * @param template Template to base the comparison on
 */
exports.like = function (template) { return ({
    'pact:matcher:type': 'type',
    value: template,
}); };
/**
 * Object where the key itself is ignored, but the value template must match.
 *
 * @param keyTemplate Example key to use
 * @param template Example value template to base the comparison on
 */
exports.eachKeyLike = function (keyTemplate, template) {
    var _a;
    return ({
        'pact:matcher:type': 'values',
        value: (_a = {},
            _a[keyTemplate] = template,
            _a),
    });
};
/**
 * Array where each element must match the given template
 * @param template Template to base the comparison on
 */
exports.eachLike = function (template) { return ({
    'pact:matcher:type': 'type',
    value: [template],
}); };
/**
 * An array that has to have at least one element and each element must match the given template
 * @param template Template to base the comparison on
 * @param count Number of examples to generate, defaults to one
 */
exports.atLeastOneLike = function (template, count) {
    if (count === void 0) { count = 1; }
    return ({
        min: 1,
        'pact:matcher:type': 'type',
        value: ramda_1.times(function () { return template; }, count),
    });
};
/**
 * An array that has to have at least the required number of elements and each element must match the given template
 * @param template Template to base the comparison on
 * @param min Minimum number of elements required in the array
 * @param count Number of examples to generate, defaults to min
 */
exports.atLeastLike = function (template, min, count) {
    var elements = count || min;
    if (count && count < min) {
        throw new Error("atLeastLike has a minimum of " + min + " but " + count + " elements were requested." +
            " Make sure the count is greater than or equal to the min.");
    }
    return {
        min: min,
        'pact:matcher:type': 'type',
        value: ramda_1.times(function () { return template; }, elements),
    };
};
/**
 * An array that has to have at most the required number of elements and each element must match the given template
 * @param template Template to base the comparison on
 * @param max Maximum number of elements required in the array
 * @param count Number of examples to generate, defaults to one
 */
exports.atMostLike = function (template, max, count) {
    var elements = count || 1;
    if (count && count > max) {
        throw new Error("atMostLike has a maximum of " + max + " but " + count + " elements where requested." +
            " Make sure the count is less than or equal to the max.");
    }
    return {
        max: max,
        'pact:matcher:type': 'type',
        value: ramda_1.times(function () { return template; }, elements),
    };
};
/**
 * An array whose size is constrained to the minimum and maximum number of elements and each element must match the given template
 * @param template Template to base the comparison on
 * @param min Minimum number of elements required in the array
 * @param max Maximum number of elements required in the array
 * @param count Number of examples to generate, defaults to one
 */
exports.constrainedArrayLike = function (template, min, max, count) {
    var elements = count || min;
    if (count) {
        if (count < min) {
            throw new Error("constrainedArrayLike has a minimum of " + min + " but " + count + " elements where requested." +
                " Make sure the count is greater than or equal to the min.");
        }
        else if (count > max) {
            throw new Error("constrainedArrayLike has a maximum of " + max + " but " + count + " elements where requested." +
                " Make sure the count is less than or equal to the max.");
        }
    }
    return {
        min: min,
        max: max,
        'pact:matcher:type': 'type',
        value: ramda_1.times(function () { return template; }, elements),
    };
};
/**
 * Value must be a boolean
 * @param b Boolean example value. Defaults to true if unsupplied
 */
exports.boolean = function (b) {
    if (b === void 0) { b = true; }
    return ({
        'pact:matcher:type': 'type',
        value: b,
    });
};
/**
 * Value must be an integer (must be a number and have no decimal places)
 * @param int Example value. If omitted a random value will be generated.
 */
exports.integer = function (int) {
    if (int) {
        return {
            'pact:matcher:type': 'integer',
            value: int,
        };
    }
    return {
        'pact:generator:type': 'RandomInt',
        'pact:matcher:type': 'integer',
        value: 101,
    };
};
/**
 * Value must be a decimal number (must be a number and have decimal places)
 * @param num Example value. If omitted a random value will be generated.
 */
exports.decimal = function (num) {
    if (num) {
        return {
            'pact:matcher:type': 'decimal',
            value: num,
        };
    }
    return {
        'pact:generator:type': 'RandomDecimal',
        'pact:matcher:type': 'decimal',
        value: 12.34,
    };
};
/**
 * Value must be a number
 * @param num Example value. If omitted a random integer value will be generated.
 */
function number(num) {
    if (num) {
        return {
            'pact:matcher:type': 'number',
            value: num,
        };
    }
    return {
        'pact:generator:type': 'RandomInt',
        'pact:matcher:type': 'number',
        value: 1234,
    };
}
exports.number = number;
/**
 * Value must be a string
 * @param str Example value
 */
function string(str) {
    return {
        'pact:matcher:type': 'type',
        value: str,
    };
}
exports.string = string;
/**
 * Value that must match the given regular expression
 * @param pattern Regular Expression to match
 * @param str Example value
 */
function regex(pattern, str) {
    if (pattern instanceof RegExp) {
        return {
            'pact:matcher:type': 'regex',
            regex: pattern.source,
            value: str,
        };
    }
    return {
        'pact:matcher:type': 'regex',
        regex: pattern,
        value: str,
    };
}
exports.regex = regex;
/**
 * Value that must be equal to the example. This is mainly used to reset the matching rules which cascade.
 * @param value Example value
 */
exports.equal = function (value) { return ({
    'pact:matcher:type': 'equality',
    value: value,
}); };
/**
 * String value that must match the provided datetime format string.
 * @param format Datetime format string. See [Java SimpleDateFormat](https://docs.oracle.com/javase/8/docs/api/java/text/SimpleDateFormat.html)
 * @param example Example value to use. If omitted a value using the current system date and time will be generated.
 */
function datetime(format, example) {
    return ramda_1.pickBy(function (v) { return !ramda_1.isNil(v); }, {
        'pact:generator:type': example ? undefined : 'DateTime',
        'pact:matcher:type': 'timestamp',
        format: format,
        value: example || index_node_1.default.generate_datetime_string(format),
    });
}
exports.datetime = datetime;
/**
 * String value that must match the provided datetime format string.
 * @param format Datetime format string. See [Java SimpleDateFormat](https://docs.oracle.com/javase/8/docs/api/java/text/SimpleDateFormat.html)
 * @param example Example value to use. If omitted a value using the current system date and time will be generated.
 */
function timestamp(format, example) {
    return datetime(format, example);
}
exports.timestamp = timestamp;
/**
 * String value that must match the provided time format string.
 * @param format Time format string. See [Java SimpleDateFormat](https://docs.oracle.com/javase/8/docs/api/java/text/SimpleDateFormat.html)
 * @param example Example value to use. If omitted a value using the current system time will be generated.
 */
function time(format, example) {
    return {
        'pact:generator:type': 'Time',
        'pact:matcher:type': 'time',
        format: format,
        value: example || index_node_1.default.generate_datetime_string(format),
    };
}
exports.time = time;
/**
 * String value that must match the provided date format string.
 * @param format Date format string. See [Java SimpleDateFormat](https://docs.oracle.com/javase/8/docs/api/java/text/SimpleDateFormat.html)
 * @param example Example value to use. If omitted a value using the current system date will be generated.
 */
function date(format, example) {
    return {
        format: format,
        'pact:generator:type': 'Date',
        'pact:matcher:type': 'date',
        value: example || index_node_1.default.generate_datetime_string(format),
    };
}
exports.date = date;
/**
 * Value that must include the example value as a substring.
 * @param value String value to include
 */
function includes(value) {
    return {
        'pact:matcher:type': 'include',
        value: value,
    };
}
exports.includes = includes;
/**
 * Value that must be null. This will only match the JSON Null value. For other content types, it will
 * match if the attribute is missing.
 */
function nullValue() {
    return {
        'pact:matcher:type': 'null',
    };
}
exports.nullValue = nullValue;
/**
 * Matches a URL composed of a base path and a list of path fragments
 * @param basePath Base path of the URL. If null, will use the base URL from the mock server.
 * @param pathFragments list of path fragments, can be regular expressions
 */
function url2(basePath, pathFragments) {
    var regexpr = __spreadArrays([
        '.*('
    ], pathFragments.map(function (p) {
        if (p instanceof RegExp) {
            return "\\/" + p.source;
        }
        if (p instanceof Object && p['pact:matcher:type'] === 'regex') {
            return "\\/" + p.regex;
        }
        return "\\/" + p.toString();
    })).join('');
    var example = __spreadArrays([
        basePath || 'http://localhost:8080'
    ], pathFragments.map(function (p) {
        if (p instanceof RegExp) {
            return "/" + index_node_1.default.generate_regex_string(p.source);
        }
        if (p instanceof Object && p['pact:matcher:type'] === 'regex') {
            return "/" + p.value;
        }
        return "/" + p.toString();
    })).join('');
    // Temporary fix for inconsistancies between matchers and generators. Matchers use "value" attribute for
    // example values, while generators use "example"
    if (basePath == null) {
        return {
            'pact:matcher:type': 'regex',
            'pact:generator:type': 'MockServerURL',
            regex: regexpr + ")$",
            value: example,
            example: example,
        };
    }
    return {
        'pact:matcher:type': 'regex',
        regex: regexpr + ")$",
        value: example,
    };
}
exports.url2 = url2;
/**
 * Matches a URL composed of a list of path fragments. The base URL from the mock server will be used.
 * @param pathFragments list of path fragments, can be regular expressions
 */
function url(pathFragments) {
    return url2(null, pathFragments);
}
exports.url = url;
/**
 * Matches the items in an array against a number of variants. Matching is successful if each variant
 * occurs once in the array. Variants may be objects containing matching rules.
 */
function arrayContaining() {
    var variants = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        variants[_i] = arguments[_i];
    }
    return {
        'pact:matcher:type': 'arrayContains',
        variants: variants,
    };
}
exports.arrayContaining = arrayContaining;
/**
 * Marks an item to be injected from the provider state
 * @param expression Expression to lookup in the provider state context
 * @param exampleValue Example value to use in the consumer test
 */
function fromProviderState(expression, exampleValue) {
    return {
        'pact:matcher:type': 'type',
        'pact:generator:type': 'ProviderState',
        expression: expression,
        value: exampleValue,
    };
}
exports.fromProviderState = fromProviderState;
/**
 * Match a universally unique identifier (UUID). Random values will be used for examples if no example is given.
 */
function uuid(example) {
    var regexStr = '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}';
    if (example) {
        var regexpr = new RegExp("^" + regexStr + "$");
        if (!example.match(regexpr)) {
            throw new Error("regex: Example value '" + example + "' does not match the UUID regular expression '" + regexStr + "'");
        }
        return {
            'pact:matcher:type': 'regex',
            regex: regexStr,
            value: example,
        };
    }
    return {
        'pact:matcher:type': 'regex',
        regex: regexStr,
        'pact:generator:type': 'Uuid',
        value: 'e2490de5-5bd3-43d5-b7c4-526e33f71304',
    };
}
exports.uuid = uuid;
//# sourceMappingURL=matchers.js.map